@@ -39,12 +39,11 @@ const { async_id_symbol } = require('internal/async_hooks').symbols;
 const {
   codes: {
     ERR_INVALID_ARG_TYPE,
-    ERR_INVALID_OPT_VALUE,
     ERR_OUT_OF_RANGE,
   },
 } = require('internal/errors');
 const { once } = require('internal/util');
-const { validateNumber } = require('internal/validators');
+const { validateNumber, validateOneOf } = require('internal/validators');
 
 const kOnKeylog = Symbol('onkeylog');
 const kRequestOptions = Symbol('requestOptions');
@@ -99,9 +98,7 @@ function Agent(options) {
   this.maxTotalSockets = this.options.maxTotalSockets;
   this.totalSocketCount = 0;
 
-  if (this.scheduling !== 'fifo' && this.scheduling !== 'lifo') {
-    throw new ERR_INVALID_OPT_VALUE('scheduling', this.scheduling);
-  }
+  validateOneOf(this.scheduling, 'scheduling', ['fifo', 'lifo'], true);
 
   if (this.maxTotalSockets !== undefined) {
     validateNumber(this.maxTotalSockets, 'maxTotalSockets');
@@ -438,7 +435,7 @@ Agent.prototype.removeSocket = function removeSocket(s, options) {
     // There might be older requests in a different origin, but
     // if the origin which releases the socket has pending requests
     // that will be prioritized.
-    for (const prop in this.requests) {
+    for (const prop of ObjectKeys(this.requests)) {
       // Check whether this specific origin is already at maxSockets
       if (this.sockets[prop] && this.sockets[prop].length) break;
       debug('removeSocket, have a request with different origin,' +
