@@ -67,8 +67,7 @@ fs.readdir(__filename, {
 // Check the readdir async version
 fs.readdir(readdirDir, {
   withFileTypes: true
-}, common.mustCall((err, dirents) => {
-  assert.ifError(err);
+}, common.mustSucceed((dirents) => {
   assertDirents(dirents);
 }));
 
@@ -104,8 +103,7 @@ binding.readdir = common.mustCall((path, encoding, types, req, ctx) => {
 assertDirents(fs.readdirSync(readdirDir, { withFileTypes: true }));
 fs.readdir(readdirDir, {
   withFileTypes: true
-}, common.mustCall((err, dirents) => {
-  assert.ifError(err);
+}, common.mustSucceed((dirents) => {
   assertDirents(dirents);
 }));
 
